## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "#>"
)

## ----echo=FALSE---------------------------------------------------------------
library(LimROTS)
data("UPS1.Case4")
table(UPS1.Case4$fake.batch, UPS1.Case4$Conc.)

## -----------------------------------------------------------------------------
# Load necessary packages
library(LimROTS)
library(parallel)
library(ggplot2)

## -----------------------------------------------------------------------------
# Load the dataset
data("UPS1.Case4")
print(UPS1.Case4)

## -----------------------------------------------------------------------------
set.seed(1234, kind = "default")

## -----------------------------------------------------------------------------
# Set metadata and formula for LimROTS analysis
meta.info <- c("Conc.", "tool", "fake.batch")
B <- 100 # Number of bootstrap samples
K <- 100 # Set the value for K based on the data size
K <- floor(K)
num_cores <- 2 # Number of cores for parallel processing
cluster <- makeCluster(num_cores) # Create a parallel cluster
group.name <- "Conc."
formula.str <- "~ 0 + Conc. + tool + fake.batch" # Formula for group comparison

# Run LimROTS analysis with trend and robust settings enabled
limrots.result <- LimROTS(
    x = UPS1.Case4,
    B = B, K = K, meta.info = meta.info,
    cluster = cluster, group.name = group.name,
    formula.str = formula.str, trend = TRUE, 
    robust = TRUE, permutating.group = FALSE, 
    seed.cl = 1234
)

## -----------------------------------------------------------------------------
# Create a data frame from the LimROTS results
limrots.result.df <- data.frame(
    proteins = row.names(limrots.result$data),
    LimROTS.FC = limrots.result$corrected.logf,
    q.value = limrots.result$q_values$qvalues
)
# Mark proteins as true positives (HUMAN UPS1 proteins)
limrots.result.df$TP <- ifelse(grepl("HUMAN", limrots.result.df$proteins),
    "HUMAN_TP", "ECOLI_FP"
)

# Create a volcano plot
ggplot(limrots.result.df, aes(
    x = LimROTS.FC, y = -log10(q.value),
    color = factor(TP)
)) +
    geom_point(alpha = 0.8) +
    theme_bw() +
    labs(
        title = "Volcano Plot", x = "Log Fold Change", y = "-Log10 q.value",
        color = "True Positive"
    ) +
    scale_color_manual(values = c("grey", "red")) +
    geom_hline(yintercept = -log10(0.05), linetype = "dashed", color = "blue")+
    geom_hline(yintercept = -log10(0.01), linetype = "dashed", color = "black")

## ----results="hide" , message=FALSE, warning=FALSE----------------------------
## Quality Control Plots

# Plot of q-values
plot(limrots.result$q_values,
    main = "Q-values", xlab = "Index",
    ylab = "Q-value"
)

# Histogram of q-values
hist(limrots.result$q_values,
    main = "Q-value Distribution",
    xlab = "Q-value", col = "lightgreen", border = "white"
)

# Summary of q-values
summary(limrots.result$q_values)

## -----------------------------------------------------------------------------
sessionInfo()

